/*
 *  ntfs.h
 *
 *  Copyright (C) 1999 Standard & Western Software
 *  http://www.sw-soft.com
 *  This software is distributed AS IS without
 *  warranty of any kind.
 *
 */
 
#ifndef _NTFS_H
#define _NTFS_H

#ifdef __cplusplus
extern "C" {
#endif

#include "fsproto.h"
#include "libntfs/ntfstypes.h"
#include "libntfs/struct.h"
#include "libntfs/support.h"
#include "libntfs/util.h"
#include "libntfs/attr.h"
#include "libntfs/dir.h"
#include "libntfs/inode.h"
#include "libntfs/macros.h"


/* For each item on the disk (directory, file, etc), your filesystem should allocate a vnode struct and 
	pass it back to the kernel when fs_read_vnode is called. This struct is then passed back in to 
	your file system by functions that reference an item on the disk. You'll need to be able to
	create a vnode from a vnode id, either by hashing the id number or encoding the information needed
	to create the vnode in the vnode id itself. Vnode ids are assigned by your file system when the
	filesystem walk function is called. For this driver, the block number is encoded in the upper bits
	of the vnode id, and the offset within the block in the lower, allowing me to just read the info
	to fill in the vnode struct from the disk. When the kernel is done with a vnode, it will call 
	fs_write_vnode (somewhat of a misnomer) where you should deallocate the struct.
*/

typedef struct vnode
{
	ntfs_inode	ntnode;
	int			parent_number;
} vnode;

#define NTFS_ROOTNODE_ID 5

/* Structure used for directory "cookie". When you are asked
 	to open a directory, you are asked to create a cookie for it
 	and pass it back. The cookie should contain the information you
 	need to determine where you are at in reading the directory 
 	entries, incremented every time readdir is called, until finally
 	the end is reached, when readdir returns NULL. */

typedef struct dircookie
{
	ntfs_iterate_s	walk;
	int				first;
	int				state;
} dircookie;
 
/* You may also need to define a cookie for files, which again is 
	allocated every time a file is opened and freed when the free
	cookie function is called. For ISO, we didn't need one.
*/

/* This is the global volume nspace struct. When mount is called , this struct should
	be allocated. It is passed back into the functions so that you can get at any
	global information you need. You'll need to change this struct to suit your purposes.
*/

typedef struct nspace
{
	ntfs_volume	*ntvol;
	char		devicePath[127];
	nspace_id	id;
	int			free_cluster_count;
	char		volumeLabel[127];
} nspace;

int NTFSMount( char *path, int flags, nspace **newVol );
int NTFSReadVolumeLabel( nspace *ns );
int	NTFSReadDirEnt( nspace *ns, dircookie *cookie, struct dirent *buf, size_t bufsize );

#ifdef __cplusplus
}
#endif


#endif
